@@ -1,6 +1,6 @@
 expcomp <- function(object, ...)
 { 	 		
-  fList <- list(b3, b4, b5, b6, l3, l4, l5, l6)
+  fList <- list(b3, b4, b5, b6, b7, l3, l4, l5, l6, l7)
 	fnList <- lapply(fList, function(x) x$name)
   print("Fitting all sigmoidal models...")
   flush.console()
@@ -27,4 +27,4 @@ expcomp <- function(object, ...)
 		else plot(modList[[i]], add = TRUE, col = cols[i], lwd = lwds[i])
 	}
 	return(cbind(model = fnList[order(rk)], RMSE = RMSEs[order(rk)]))
-}	
\ No newline at end of file
+}	
